@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2018  Vincent Pelletier <plr.vincent@gmail.com>
+# Copyright (C) 2016-2020  Vincent Pelletier <plr.vincent@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -77,22 +77,19 @@ Example statistic usage (to profile other running threads):
     )
     return data
 """
-
+from __future__ import print_function
 import dis
-from email.mime.multipart import MIMEMultipart
-from email.mime.text import MIMEText
-from email.mime.application import MIMEApplication
-from email.encoders import encode_quopri
 import functools
 import gc
 from io import StringIO, BytesIO
 from importlib import import_module
 import itertools
-import os
 from collections import defaultdict
-import zipfile
 import pprofile
 
+if sys.version_info >= (3, ):
+    unicode = str
+
 def getFuncCodeOrNone(module, attribute_path):
     try:
         value = import_module(module)
@@ -127,7 +124,6 @@ else:
     functools.update_wrapper(unrestrictedTraverse_spy, orig_unrestrictedTraverse)
     OFS.Traversable.Traversable.unrestrictedTraverse = unrestrictedTraverse_spy
 
-_ALLSEP = os.sep + (os.altsep or '')
 PYTHON_EXPR_FUNC_CODE_SET = (ZRPythonExpr__call__func_code, PythonExpr__call__func_code)
 
 class ZopeFileTiming(pprofile.FileTiming):
@@ -163,12 +159,12 @@ def tabulate(title_list, row_list):
     for row in row_list:
         assert len(row) == column_count, repr(row)
         for index, value in enumerate(row):
-            max_width_list[index] = max(max_width_list[index], len(str(value)))
-    format_string = ''.join('| %%-%is ' % x for x in max_width_list) + '|\n'
+            max_width_list[index] = max(max_width_list[index], len(unicode(value)))
+    format_string = u''.join(u'| %%-%is ' % x for x in max_width_list) + u'|\n'
     out = StringIO()
     write = out.write
     write(format_string % tuple(title_list))
-    write(''.join('+' + ('-' * (x + 2)) for x in max_width_list) + '+\n')
+    write(u''.join(u'+' + (u'-' * (x + 2)) for x in max_width_list) + u'+\n')
     for row in row_list:
         write(format_string % tuple(row))
     return out.getvalue()
@@ -193,21 +189,21 @@ def disassemble(co, lasti=-1):
         op = ord(c)
         if i in linestarts:
             if i > 0:
-                print(end='\n', file=out)
-            print('%3d' % linestarts[i], end=' ', file=out)
+                print(end=u'\n', file=out)
+            print(u'%3d' % linestarts[i], end=u' ', file=out)
         else:
-            print('   ', end=' ', file=out)
+            print(u'   ', end=u' ', file=out)
 
         if i == lasti:
-            print('-->', end=' ', file=out)
+            print(u'-->', end=u' ', file=out)
         else:
-            print('   ', end=' ', file=out)
+            print(u'   ', end=u' ', file=out)
         if i in labels:
-            print('>>', end=' ', file=out)
+            print(u'>>', end=u' ', file=out)
         else:
-            print('  ', end=' ', file=out)
-        print('%4i' % i, end=' ', file=out)
-        print('%-20s' % dis.opname[op], end=' ', file=out)
+            print(u'  ', end=u' ', file=out)
+        print(u'%4i' % i, end=u' ', file=out)
+        print(u'%-20s' % dis.opname[op], end=u' ', file=out)
         i = i + 1
         if op >= dis.HAVE_ARGUMENT:
             oparg = ord(code[i]) + ord(code[i + 1]) * 256 + extended_arg
@@ -215,22 +211,22 @@ def disassemble(co, lasti=-1):
             i = i + 2
             if op == dis.EXTENDED_ARG:
                 extended_arg = oparg * 65536
-            print('%5i' % oparg, end=' ', file=out)
+            print(u'%5i' % oparg, end=u' ', file=out)
             if op in dis.hasconst:
-                print('(%r)' % co.co_consts[oparg], end=' ', file=out)
+                print(u'(%r)' % co.co_consts[oparg], end=u' ', file=out)
             elif op in dis.hasname:
-                print('(%s)' % co.co_names[oparg], end=' ', file=out)
+                print(u'(%s)' % co.co_names[oparg], end=u' ', file=out)
             elif op in dis.hasjrel:
-                print('(to %r)' % (i + oparg), end=' ', file=out)
+                print(u'(to %r)' % (i + oparg), end=u' ', file=out)
             elif op in dis.haslocal:
-                print('(%s)' % co.co_varnames[oparg], end=' ', file=out)
+                print(u'(%s)' % co.co_varnames[oparg], end=u' ', file=out)
             elif op in dis.hascompare:
-                print('(%s)' % dis.cmp_op[oparg], end=' ', file=out)
+                print(u'(%s)' % dis.cmp_op[oparg], end=u' ', file=out)
             elif op in dis.hasfree:
                 if free is None:
                     free = co.co_cellvars + co.co_freevars
-                print('(%s)' % free[oparg], end=' ', file=out)
-        print(end='\n', file=out)
+                print(u'(%s)' % free[oparg], end=u' ', file=out)
+        print(end=u'\n', file=out)
     return out.getvalue()
 
 class ZopeMixIn(object):
@@ -338,9 +334,9 @@ class ZopeMixIn(object):
                             None,
                         ) is SharedDCScriptsBindings_bindAndExec_func_code:
                             name = self._rememberFile(
-                                '# This is an auto-generated preamble executed '
-                                'by Shared.DC.Scripts.Bindings before "actual" '
-                                'code.\n' +
+                                u'# This is an auto-generated preamble executed '
+                                u'by Shared.DC.Scripts.Bindings before "actual" '
+                                u'code.\n' +
                                 disassemble(frame.f_code),
                                 'preamble',
                                 '.py.bytecode',
@@ -349,7 +345,7 @@ class ZopeMixIn(object):
                             # Could not find source, provide disassembled
                             # bytecode as last resort.
                             name = self._rememberFile(
-                                '# Unidentified source for ' +
+                                u'# Unidentified source for ' +
                                 name + '\n' +
                                 disassemble(
                                     frame.f_code,
@@ -376,7 +372,7 @@ class ZopeMixIn(object):
                         )
                     elif evaluator_code in PYTHON_EXPR_FUNC_CODE_SET:
                         source = evaluator_frame.f_locals['self'].text
-                        if not isinstance(source, str):
+                        if not isinstance(source, unicode):
                             source = source.decode('utf-8')
                         name = self._rememberFile(
                             source,
@@ -404,36 +400,19 @@ class ZopeMixIn(object):
             self.keep_alive.append(frame_globals)
             return file_timing
 
-    def _iterOutFiles(self):
+    def _iterOutFiles(self, *args, **kw):
         """
         Yields path, data, mimetype for each file involved on or produced by
         profiling.
         """
-        out = StringIO()
-        self.callgrind(out, relative_path=True)
-        yield (
-            'cachegrind.out.pprofile',
-            out.getvalue(),
-            'application/x-kcachegrind',
-        )
-        for name, lines in self.iterSource():
-            lines = ''.join(lines)
-            if lines:
-                if isinstance(lines, str):
-                    lines = lines.encode('utf-8')
-                yield (
-                    os.path.normpath(
-                        os.path.splitdrive(name)[1]
-                    ).lstrip(_ALLSEP),
-                    lines,
-                    'text/x-python',
-                )
+        for entry in super(ZopeMixIn, self)._iterOutFiles(*args, **kw):
+            yield entry
         sql_name_template = 'query_%%0%ii-%%i_hits_%%6fs.sql' % len(
             str(len(self.sql_dict)),
         )
         for index, (query, time_list) in enumerate(
             sorted(
-                iter(self.sql_dict.items()),
+                self.sql_dict.items(),
                 key=lambda x: (sum(x[1]), len(x[1])),
                 reverse=True,
             ),
@@ -485,7 +464,7 @@ class ZopeMixIn(object):
                 'text/plain',
             )
 
-    def asMIMEString(self):
+    def getCallgrindMIME(self):
         """
         Return a mime-multipart representation of:
         - callgrind profiling statistics (cachegrind.out.pprofile)
@@ -495,37 +474,17 @@ class ZopeMixIn(object):
           (unrestrictedTraverse_pathlist.txt)
         - all involved python code, including Python Scripts without hierarchy
           (the rest)
+        and the mimetype of this string.
         To unpack resulting file, see "unpack a MIME message" in
           http://docs.python.org/2/library/email-examples.html
         Or get demultipart from
           https://pypi.python.org/pypi/demultipart
         """
-        result = MIMEMultipart()
-        base_type_dict = {
-            'application': MIMEApplication,
-            'text': MIMEText,
-        }
-        encoder_dict = {
-            'application/x-kcachegrind': encode_quopri,
-            'text/x-python': 'utf-8',
-            'text/plain': 'utf-8',
-        }
-        for path, data, mimetype in self._iterOutFiles():
-            base_type, sub_type = mimetype.split('/')
-            chunk = base_type_dict[base_type](
-                data,
-                sub_type,
-                encoder_dict.get(mimetype),
-            )
-            chunk.add_header(
-                'Content-Disposition',
-                'attachment',
-                filename=path,
-            )
-            result.attach(chunk)
-        return result.as_string(), result['content-type']
+        out = BytesIO()
+        mimetype = super(ZopeMixIn, self).getCallgrindMIME(out)
+        return out.getvalue(), mimetype
 
-    def asZip(self):
+    def getCallgrindZip(self):
         """
         Return a serialised zip archive containing:
         - callgrind profiling statistics (cachegrind.out.pprofile)
@@ -535,16 +494,15 @@ class ZopeMixIn(object):
           (unrestrictedTraverse_pathlist.txt)
         - all involved python code, including Python Scripts without hierarchy
           (the rest)
+        and the "application/zip" mimetype.
         """
         out = BytesIO()
-        with zipfile.ZipFile(
-            out,
-            mode='w',
-            compression=zipfile.ZIP_DEFLATED,
-        ) as outfile:
-            for path, data, _ in self._iterOutFiles():
-                outfile.writestr(path, data)
-        return out.getvalue(), 'application/zip'
+        mimetype = super(ZopeMixIn, self).getCallgrindZip(out)
+        return out.getvalue(), mimetype
+
+    # BBB
+    asMIMEString = getCallgrindMIME
+    asZip = getCallgrindZip
 
 class ZopeProfiler(ZopeMixIn, pprofile.Profile):
     __slots__ = ZopeMixIn.virtual__slots__
